/*
 * Copyright (C) 2013 John Graham <johngavingraham@googlemail.com>.
 *
 * This file is part of Pthread Pool.
 *
 * Pthread Pool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * Pthread Pool is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Pthread Pool.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

/*
 * Note: This is the test included in the pthread_pool_create.3 man
 * page - if you change it, remember to change it in the man page (and
 * to turn all the "\n"s into "\\n"s.
 */

#include <pthread_pool.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define NUM_TASKS   10
#define NUM_WORKERS 2

/* Our tasks - simply return the increment of the argument. */
void *task(void *arg)
{
  return 1 + (char *)arg;
}

int main(void)
{
  pthread_pool_t pool;
  pthread_pool_task_t tasks[NUM_TASKS];
  pthread_pool_worker_t workers[NUM_WORKERS];
  pthread_pool_task_t *task_out;
  void *arg;
  int i, err;

  /* Create a pool with default attributes. */
  err = pthread_pool_create(&pool, NULL);
  if (err)
  {
    fprintf(stderr, "pthread_pool_create(): %s\n",
            strerror(err));
    return EXIT_FAILURE;
  }
  
  /* Add two more workers. */
  for (i = 0 ; i < NUM_WORKERS ; i++)
  {
    err = pthread_pool_worker_init(&workers[i], &pool, NULL);
    if (0 != err)
    {
      fprintf(stderr, "pthread_pool_worker_add(): %s (continuing)\n",
              strerror(err));
      return EXIT_FAILURE;
    }
  }
  
  /* Add our tasks. */
  for (arg = 0 ; arg < (void *)NUM_TASKS ; arg = 1 + (char *)arg) {
    err = pthread_pool_task_init(&tasks[(int)arg], &pool, &task, arg);
    if (0 != err)
    {
      fprintf(stderr, "pthread_pool_task_add(): %s\n",
              strerror(err));
      return EXIT_FAILURE;
    }
  }

  /* Wait for all tasks to finish. */
  pthread_pool_wait(&pool);
  
  /* All tasks will have finished, so if we keep trying to join
   * we'll eventually collect the results of all the tasks. */
  while (0 == pthread_pool_tryjoin(&pool, &task_out))
  {
    void *arg, *retval;
    pthread_pool_task_getarg(task_out, &arg);
    pthread_pool_task_getretval(task_out, &retval);
    printf("Input: %d\tOutput: %d\n",
           (intptr_t)arg, (intptr_t)retval);
  }
  
  pthread_pool_destroy(&pool);

  return 0;
}





























