/*
 * Copyright (C) 2013 John Graham <johngavingraham@googlemail.com>.
 *
 * This file is part of Pthread Pool.
 *
 * Pthread Pool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * Pthread Pool is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Pthread Pool.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <pthread_pool.h>

#include <errno.h>
#include <stdio.h>

#include <unistd.h>

#include <test.h>

#define NUM_TASKS   10
#define NUM_WORKERS 3

void *task(void *arg)
{
  int num = (int)arg;

  /*usleep(num);*/

  return (void *)(num + 100);
}

int main(void)
{
  int wait; /* Run once waiting, once not waiting. */

  for (wait = 0 ; wait < 2 ; wait++)
  {
    pthread_pool_t pool;
    char *arg = NULL;
    int i;
    pthread_pool_task_t tasks[NUM_TASKS];
    pthread_pool_worker_t workers[NUM_WORKERS];

    assert_equal(0, pthread_pool_create(&pool, NULL));

    if (errno)
    {
      perror("pthread_pool_create()");
      return 1;
    }

    for (i = 0 ; i < NUM_TASKS ; i++)
      pthread_pool_task_init(&tasks[i], &pool, &task, arg++);
    
    for (i = 0 ; i < NUM_WORKERS ; i++)
      assert_equal(0, pthread_pool_worker_init(&workers[i], &pool, NULL));
    
    if (wait)
      pthread_pool_wait(&pool);

    for (i = 0 ; i < 10 ; i++)
    {
      void *arg, *retval;
      pthread_pool_task_t *task;

      if (wait) {
        /* All threads should have terminated. */
        if (0 != pthread_pool_tryjoin(&pool, &task))
        {
          fprintf(stderr, "Couldn't tryjoin()\n");
          return 1;
        }
      }
      else
      {
        /* Wait for threads to terminate. */
        if (0 != pthread_pool_join(&pool, &task))
        {
          fprintf(stderr, "Couldn't join()\n");
          return 1;
        }
      }

      pthread_pool_task_getretval(task, &retval);
      pthread_pool_task_getarg(task, &arg);

      assert((int)retval == (int)arg + 100);
    }

    /* That should be all the threads. */
    if (EBUSY != pthread_pool_tryjoin(&pool, NULL))
    {
      fprintf(stderr, "Tryjoin didn't fail\n");
      return 1;
    }

    pthread_pool_destroy(&pool);
  }

  return 0;
}































